==> xdemorse_2.6.orig.tar.bz2: xdemorse-2.6/src/sound.c <==
/*  sound.c
 *
 *  Soundcard handling functions of xdemorse application
 */

/*
 *  xdemorse: An application to decode and display
 *  Morse code signals using a computer's sound card
 *
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of
 *  the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details:
 *
 *  http://www.gnu.org/copyleft/gpl.txt
 */

#include "sound.h"
#include "shared.h"

/* ALSA pcm capture and mixer handles */
static snd_pcm_t *capture_handle = NULL;
static snd_mixer_t *mixer_handle = NULL;
static snd_pcm_hw_params_t *hw_params = NULL;

/*------------------------------------------------------------------------*/

/*  Setup_Sound_Card()
 *
 *  Sets up mixer and DSP devices
 */
  gboolean
Setup_Sound_Card( char *mesg, int *error )
{
  snd_mixer_elem_t *elem;
  snd_mixer_selem_id_t *sid;
  long cmin, cmax;


  /*** Set up pcm parameters ***/
  /* Open pcm for capture */
  *error = snd_pcm_open(
	  &capture_handle, rc_data.snd_card,
	  SND_PCM_STREAM_CAPTURE, SND_PCM_ASYNC );
  if( *error < 0 )
  {
	Strlcat( mesg, _("Cannot open audio device: "),
		strlen(_("Cannot open audio device: "))+1 );
	Strlcat( mesg, rc_data.snd_card, strlen(rc_data.snd_card)+1 );
	return( FALSE );
  }

  /* Allocate memory to hardware parameters structure */
  *error = snd_pcm_hw_params_malloc( &hw_params );
  if( *error < 0 )
  {
	Strlcat( mesg,
		_("Cannot allocate hardware parameter struct"),
		strlen(_("Cannot allocate hardware parameter struct"))+1 );
	return( FALSE );
  }

  /* Initialize hardware parameter structure */
  *error = snd_pcm_hw_params_any( capture_handle, hw_params );
  if( *error < 0 )
  {
	Strlcat( mesg,
		_("Cannot initialize hardware parameter struct"),
		strlen(_("Cannot initialize hardware parameter struct"))+1 );
	return( FALSE );
  }

  /* Set access type */
  *error = snd_pcm_hw_params_set_access(
	  capture_handle, hw_params, SND_PCM_ACCESS_RW_INTERLEAVED );
  if( *error < 0 )
  {
	Strlcat( mesg,
		_("Cannot set access type (RW_INTERLEAVED)"),
		strlen(_("Cannot set access type (RW_INTERLEAVED)"))+1 );
	return( FALSE );
  }

  /* Set sample format */
  *error = snd_pcm_hw_params_set_format(
	  capture_handle, hw_params, SND_PCM_FORMAT_S16_LE );
  if( *error < 0 )
  {
	Strlcat( mesg, _("Cannot set sample format (S16_LE)"),
		strlen(_("Cannot set sample format (S16_LE)"))+1 );
	return( FALSE );
  }

  /* Set sample rate */
  *error = snd_pcm_hw_params_set_rate(
	  capture_handle, hw_params, (unsigned int)rc_data.dsp_rate, EXACT_VAL );
  if( *error < 0 )
  {
	snprintf( mesg, 127,
		_("Cannot set sample rate to %d"), rc_data.dsp_rate );
	return( FALSE );
  }

  /* Set channel count */
  *error = snd_pcm_hw_params_set_channels(
	  capture_handle, hw_params, (unsigned int)rc_data.num_chnl );
  if( *error < 0 )
  {
	snprintf( mesg, 127,
		_("Cannot set channel count to %d"), rc_data.num_chnl );
	return( FALSE );
  }

  /* Set number of periods */
  *error = snd_pcm_hw_params_set_periods(
	  capture_handle, hw_params, NUM_PERIODS, EXACT_VAL );
  if( *error < 0)
  {
	snprintf( mesg, 127,
		_("Cannot set number of periods to %d"), NUM_PERIODS );
	return( FALSE );
  }

  /* Set period size */
  *error = snd_pcm_hw_params_set_period_size(
	  capture_handle, hw_params, PERIOD_SIZE, EXACT_VAL );
  if( *error < 0)
  {
	snprintf( mesg, 127, _("Cannot set period size to %d"), PERIOD_SIZE );
	return( FALSE );
  }

  /* Set parameters */
  *error = snd_pcm_hw_params( capture_handle, hw_params );
  if( *error < 0 )
  {
	Strlcat( mesg,
		_("Cannot set capture parameters"),
		strlen(_("Cannot set capture parameters"))+1 );
	return( FALSE );
  }
  snd_pcm_hw_params_free( hw_params );
  hw_params = NULL;

  /*** Set up mixer (capture source and volume) ***/
  /* Open mixer */
  *error = snd_mixer_open( &mixer_handle, 0 );
  if( *error < 0 )
  {
	Strlcat( mesg, _("Cannot open mixer"),
		strlen(_("Cannot open mixer"))+1 );
	return( FALSE );
  }

  /* Attach mixer */
  *error = snd_mixer_attach( mixer_handle, rc_data.snd_card );
  if( *error < 0 )
  {
	Strlcat( mesg, _("Cannot attach mixer to: "),
		strlen(_("Cannot attach mixer to: "))+1 );
	Strlcat( mesg, rc_data.snd_card, strlen(rc_data.snd_card)+1 );
	return( FALSE );
  }

  /* Register mixer */
  *error = snd_mixer_selem_register( mixer_handle, NULL, NULL );
  if( *error < 0 )
  {
	Strlcat( mesg, _("Cannot register mixer"),
		strlen(_("Cannot register mixer"))+1 );
	return *error;
  }

  /* Load mixer */
  *error = snd_mixer_load( mixer_handle );
  if( *error < 0 )
  {
	Strlcat( mesg, _("Cannot load mixer"),
		strlen(_("Cannot load mixer"))+1 );
	return( FALSE );
  }

  /* Allocate selem_id structure */
  *error = snd_mixer_selem_id_malloc( &sid );
  if( *error < 0 )
  {
	Strlcat( mesg, _("Cannot allocate selem_id struct"),
		strlen(_("Cannot allocate selem_id struct"))+1 );
	return( FALSE );
  }

  /* Find capture selem */
  snd_mixer_selem_id_set_index( sid, 0 );
  snd_mixer_selem_id_set_name( sid, rc_data.cap_src );
  elem = snd_mixer_find_selem( mixer_handle, sid );
  if( !elem )
  {
	Strlcat( mesg, _("Cannot find capture element: "),
		strlen(_("Cannot find capture element: "))+1 );
	Strlcat( mesg, rc_data.cap_src, strlen(rc_data.cap_src)+1 );
	snd_mixer_selem_id_free(sid);
	*error = 0;
	return( FALSE );
  }

  /* Set capture switch for cap source */
  if( snd_mixer_selem_has_capture_switch(elem) )
  {
	*error = snd_mixer_selem_set_capture_switch(
		elem, rc_data.channel, 1 );
	if( *error < 0 )
	{
	  Strlcat( mesg, _("Cannot set capture device: "),
		  strlen(_("Cannot set capture device: "))+1 );
	  Strlcat( mesg, rc_data.cap_src, strlen(rc_data.cap_src)+1 );
	  snd_mixer_selem_id_free(sid);
	  return( FALSE );
	}
  }
  else
  {
	Strlcat( mesg, _("Device has no capture switch: "),
		strlen(_("Device has no capture switch: "))+1 );
	Strlcat( mesg, rc_data.cap_src, strlen(rc_data.cap_src)+1 );
	snd_mixer_selem_id_free(sid);
	*error = 0;
	return( FALSE );
  }

  /* Find capture volume selem */
  if( strcmp(rc_data.cap_vol, "--") != 0 )
  {
	snd_mixer_selem_id_set_index( sid, 0 );
	snd_mixer_selem_id_set_name( sid, rc_data.cap_vol );
	elem = snd_mixer_find_selem( mixer_handle, sid );
	if( elem == NULL )
	{
	  Strlcat( mesg, _("Cannot find Volume element: "),
		  strlen(_("Cannot find Volume element: "))+1 );
	  Strlcat( mesg, rc_data.cap_vol, strlen(rc_data.cap_vol)+1 );
	  Error_Dialog( mesg );
	}

	/* Set capture volume */
	if( elem != NULL )
	{
	  if( snd_mixer_selem_has_capture_volume(elem) )
	  {
		/* Change from % volume to sound card value */
		long lev;
		snd_mixer_selem_get_capture_volume_range( elem, &cmin, &cmax );
		lev = cmin + ((cmax - cmin) * rc_data.cap_level) / 100;

		/* Set capture volume */
		*error = snd_mixer_selem_set_capture_volume(
			elem, rc_data.channel, lev );
		if( *error < 0 )
		{
		  snprintf( mesg, 127,
			  _("Cannot set capture volume to %d\n"\
				"Error: %s"),
			  rc_data.cap_level, snd_strerror(*error) );
		  Error_Dialog( mesg );
		}
	  }
	  else
	  {
		Strlcat( mesg,
			_("Element has no Volume control: "),
			strlen(_("Element has no Volume control: "))+1 );
		Strlcat( mesg, rc_data.cap_vol, strlen(rc_data.cap_vol)+1 );
		*error = 0;
		Error_Dialog( mesg );
	  }
	} /* if( elem != NULL ) */

  } /* if( strcmp(rc_data.cap_vol, "--") != 0 ) */

  snd_mixer_selem_id_free(sid);
  Close_Mixer_Handle();

  /* Prepare audio interface for use */
  *error = snd_pcm_prepare( capture_handle );
  if( *error < 0 )
  {
	Strlcat( mesg,
		_("Cannot prepare audio interface"),
		strlen(_("Cannot prepare audio interface"))+1 );
	return( FALSE );
  }

  return( TRUE );
} /* End of Setup_Sound_Card() */

/*------------------------------------------------------------------------*/

/*
 * These two functions close the sound card handles
 */
  void
Close_Capture_Handle( void )
{
  if( capture_handle != NULL )
	snd_pcm_close( capture_handle );
  capture_handle = NULL;

  if( hw_params != NULL )
	snd_pcm_hw_params_free( hw_params );
  hw_params = NULL;
}

  void
Close_Mixer_Handle( void )
{
  if( mixer_handle != NULL )
	snd_mixer_close( mixer_handle );
  mixer_handle = NULL;
}

/*------------------------------------------------------------------------*/

/*  Get_Signal_Sample()
 *
 *  Gets the next DSP sample of the signal input
 */

  gboolean
Signal_Sample( short *sample )
{
  snd_pcm_sframes_t error;

  static int
	fft_idx = 0, /* fft input buffer idx */
	cnt = 0;	 /* Count of calls to this function */


  /* Refill dsp samples buffer when needed */
  if( samples_buffer.buffer_idx >= samples_buffer.buffer_size )
  {
	/* Start buffer index according to channel */
	samples_buffer.buffer_idx = rc_data.use_chnl;

	/* Read audio samples from DSP, abort on error */
	error = snd_pcm_readi(
		capture_handle, samples_buffer.buffer,	READI_FRAMES );
	if( error != READI_FRAMES )
	{
	  char mesg[128];
	  snprintf( mesg, 127,
		  _("Read from audio interface failed\n"\
			"Error: %s\n"), snd_strerror((int)error) );
	  fprintf( stderr, mesg );

	  /* Try to recover if broken pipe or suspend */
	  if( !Xrun_Recovery((int)error) )
	  {
		Cleanup();
		return( FALSE );
	  }
	}
  } /* End of if( buffer_idx >= buffer_size ) */

  /* Get a sample from the buffer in short form */
  *sample = samples_buffer.buffer[samples_buffer.buffer_idx];

  /* Take a sample for the FFT and reset stride counter */
  if( ++cnt >= rc_data.fft_stride )
  {
	fft_in_r[fft_idx]  = *sample;
	fft_idx++;
	cnt = 0;
  }

  /* Display waterfall when input buffer full */
  if( fft_idx >= FFT_INPUT_SIZE )
  {
	fft_idx = 0;
	Display_Waterfall();
  }

  /* Increment according to mono/stereo mode */
  samples_buffer.buffer_idx += rc_data.num_chnl;

  return( TRUE );
} /* Signal_Sample() */

/*------------------------------------------------------------------------*/

/* Xrun_Recovery()
 *
 * Recover from underrun (broken pipe) and suspend
 */
  gboolean
Xrun_Recovery( int error )
{
  if( error == -EPIPE )
  {
	error = snd_pcm_prepare( capture_handle );
	if( error < 0 )
	{
	  char mesg[128];
	  snprintf( mesg, 127,
		  _("Cannot recover from underrun, prepare failed\n"\
			"Error: %s\n"), snd_strerror(error) );
	  Error_Dialog( mesg );
	  return( FALSE );
	}
  }
  else if( error == -ESTRPIPE )
  {
	while( (error = snd_pcm_resume(capture_handle)) == -EAGAIN )
	  sleep(1);
	if( error < 0 )
	{
	  error = snd_pcm_prepare( capture_handle );
	  if( error < 0 )
	  {
		char mesg[128];
		snprintf( mesg, 127,
			_("Cannot recover from suspend, prepare failed\n"\
			"Error: %s\n"), snd_strerror(error) );
		Error_Dialog( mesg );
		return( FALSE );
	  }
	}
  }

  return( TRUE );
} /* Xrun_Recovery() */

/*------------------------------------------------------------------------*/

==> xdemorse_3.4.orig.tar.bz2: xdemorse-3.4/src/sound.c <==
/*
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 3 of
 *  the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details:
 *
 *  http://www.gnu.org/copyleft/gpl.txt
 */

#include "sound.h"
#include "shared.h"

/* ALSA pcm capture and mixer handles */
static snd_pcm_t *capture_handle = NULL;
static snd_mixer_t *mixer_handle = NULL;
static snd_pcm_hw_params_t *hw_params = NULL;

/*------------------------------------------------------------------------*/

/*  Setup_Sound_Card()
 *
 *  Sets up mixer and DSP devices
 */
  gboolean
Setup_Sound_Card( char *mesg, int *error )
{
  snd_mixer_elem_t *elem;
  snd_mixer_selem_id_t *sid;
  long cmin, cmax;


  /*** Set up pcm parameters ***/
  /* Open pcm for capture */
  *error = snd_pcm_open(
	  &capture_handle, rc_data.snd_card,
	  SND_PCM_STREAM_CAPTURE, SND_PCM_ASYNC );
  if( *error < 0 )
  {
	Strlcat( mesg,
		_("Cannot open audio device: "), MESG_SIZE );
	Strlcat( mesg, rc_data.snd_card, MESG_SIZE );
	return( FALSE );
  }

  /* Allocate memory to hardware parameters structure */
  *error = snd_pcm_hw_params_malloc( &hw_params );
  if( *error < 0 )
  {
	Strlcat( mesg,
		_("Cannot allocate hardware parameter struct"),
		MESG_SIZE );
	return( FALSE );
  }

  /* Initialize hardware parameter structure */
  *error = snd_pcm_hw_params_any( capture_handle, hw_params );
  if( *error < 0 )
  {
	Strlcat( mesg,
		_("Cannot initialize hardware parameter struct"),
		MESG_SIZE );
	return( FALSE );
  }

  /* Set access type */
  *error = snd_pcm_hw_params_set_access(
	  capture_handle, hw_params, SND_PCM_ACCESS_RW_INTERLEAVED );
  if( *error < 0 )
  {
	Strlcat( mesg,
		_("Cannot set access type (RW_INTERLEAVED)"),
		MESG_SIZE );
	return( FALSE );
  }

  /* Set sample format */
  *error = snd_pcm_hw_params_set_format(
	  capture_handle, hw_params, SND_PCM_FORMAT_S16_LE );
  if( *error < 0 )
  {
	Strlcat( mesg,
		_("Cannot set sample format (S16_LE)"),
		MESG_SIZE  );
	return( FALSE );
  }

  /* Set sample rate */
  *error = snd_pcm_hw_params_set_rate( capture_handle, hw_params,
	  (unsigned int)rc_data.dsp_rate, EXACT_VAL );
  if( *error < 0 )
  {
	snprintf( mesg, MESG_SIZE,
		_("Cannot set sample rate to %d"), rc_data.dsp_rate );
	return( FALSE );
  }

  /* Set channel count */
  *error = snd_pcm_hw_params_set_channels(
	  capture_handle, hw_params, (unsigned int)rc_data.num_chnl );
  if( *error < 0 )
  {
	snprintf( mesg, MESG_SIZE,
		_("Cannot set channel count to %d"), rc_data.num_chnl );
	return( FALSE );
  }

  /* Set number of periods */
  *error = snd_pcm_hw_params_set_periods(
	  capture_handle, hw_params, NUM_PERIODS, EXACT_VAL );
  if( *error < 0)
  {
	snprintf( mesg, MESG_SIZE,
		_("Cannot set number of periods to %d"), NUM_PERIODS );
	return( FALSE );
  }

  /* Set period size */
  *error = snd_pcm_hw_params_set_period_size(
	  capture_handle, hw_params, PERIOD_SIZE, EXACT_VAL );
  if( *error < 0)
  {
	snprintf( mesg, MESG_SIZE,
		_("Cannot set period size to %d"), PERIOD_SIZE );
	return( FALSE );
  }

  /* Set parameters */
  *error = snd_pcm_hw_params( capture_handle, hw_params );
  if( *error < 0 )
  {
	Strlcat( mesg, _("Cannot set capture parameters"), MESG_SIZE );
	return( FALSE );
  }
  snd_pcm_hw_params_free( hw_params );
  hw_params = NULL;

  /*** Set up mixer (capture source and volume) ***/
  /* Open mixer */
  *error = snd_mixer_open( &mixer_handle, 0 );
  if( *error < 0 )
  {
	Strlcat( mesg, _("Cannot open mixer"), MESG_SIZE );
	return( FALSE );
  }

  /* Attach mixer */
  *error = snd_mixer_attach( mixer_handle, rc_data.snd_card );
  if( *error < 0 )
  {
	Strlcat( mesg, _("Cannot attach mixer to: "), MESG_SIZE );
	Strlcat( mesg, rc_data.snd_card,  MESG_SIZE );
	return( FALSE );
  }

  /* Register mixer */
  *error = snd_mixer_selem_register( mixer_handle, NULL, NULL );
  if( *error < 0 )
  {
	Strlcat( mesg, _("Cannot register mixer"), MESG_SIZE );
	return *error;
  }

  /* Load mixer */
  *error = snd_mixer_load( mixer_handle );
  if( *error < 0 )
  {
	Strlcat( mesg, _("Cannot load mixer"), MESG_SIZE );
	return( FALSE );
  }

  /* Allocate selem_id structure */
  *error = snd_mixer_selem_id_malloc( &sid );
  if( *error < 0 )
  {
	Strlcat( mesg,
		_("Cannot allocate selem_id struct"), MESG_SIZE );
	return( FALSE );
  }

  /* Find capture selem */
  snd_mixer_selem_id_set_index( sid, 0 );
  snd_mixer_selem_id_set_name( sid, rc_data.cap_src );
  elem = snd_mixer_find_selem( mixer_handle, sid );
  if( !elem )
  {
	Strlcat( mesg,
		_("Cannot find capture element: "), MESG_SIZE );
	Strlcat( mesg, rc_data.cap_src,  MESG_SIZE );
	snd_mixer_selem_id_free(sid);
	*error = 0;
	return( FALSE );
  }

  /* Set capture switch for cap source */
  if( snd_mixer_selem_has_capture_switch(elem) )
  {
	*error = snd_mixer_selem_set_capture_switch(
		elem, rc_data.channel, 1 );
	if( *error < 0 )
	{
	  Strlcat( mesg, _("Cannot set capture device: "), MESG_SIZE );
	  Strlcat( mesg, rc_data.cap_src,  MESG_SIZE );
	  snd_mixer_selem_id_free(sid);
	  return( FALSE );
	}
  }
  else
  {
	Strlcat( mesg, _("Device has no capture switch: "), MESG_SIZE );
	Strlcat( mesg, rc_data.cap_src,  MESG_SIZE );
	snd_mixer_selem_id_free(sid);
	*error = 0;
	return( FALSE );
  }

  /* Find capture volume selem */
  if( strcmp(rc_data.cap_vol, "--") != 0 )
  {
	snd_mixer_selem_id_set_index( sid, 0 );
	snd_mixer_selem_id_set_name( sid, rc_data.cap_vol );
	elem = snd_mixer_find_selem( mixer_handle, sid );
	if( elem == NULL )
	{
	  Strlcat( mesg, _("Cannot find Volume element: "), MESG_SIZE );
	  Strlcat( mesg, rc_data.cap_vol,  MESG_SIZE );
	  Error_Dialog( mesg );
	}

	/* Set capture volume */
	if( elem != NULL )
	{
	  if( snd_mixer_selem_has_capture_volume(elem) )
	  {
		/* Change from % volume to sound card value */
		long lev;
		snd_mixer_selem_get_capture_volume_range( elem, &cmin, &cmax );
		lev = cmin + ((cmax - cmin) * rc_data.cap_level) / 100;

		/* Set capture volume */
		*error = snd_mixer_selem_set_capture_volume(
			elem, rc_data.channel, lev );
		if( *error < 0 )
		{
		  snprintf( mesg, MESG_SIZE,
			  _("Cannot set capture volume to %d\n"\
				"Error: %s"),
			  rc_data.cap_level, snd_strerror(*error) );
		  Error_Dialog( mesg );
		}
	  }
	  else
	  {
		Strlcat( mesg,
			_("Element has no Volume control: "), MESG_SIZE );
		Strlcat( mesg, rc_data.cap_vol,  MESG_SIZE );
		*error = 0;
		Error_Dialog( mesg );
	  }
	} /* if( elem != NULL ) */

  } /* if( strcmp(rc_data.cap_vol, "--") != 0 ) */

  snd_mixer_selem_id_free(sid);
  Close_Mixer_Handle();

  /* Prepare audio interface for use */
  *error = snd_pcm_prepare( capture_handle );
  if( *error < 0 )
  {
	Strlcat( mesg,	_("Cannot prepare audio interface"), MESG_SIZE );
	return( FALSE );
  }

  return( TRUE );
} /* End of Setup_Sound_Card() */

/*------------------------------------------------------------------------*/

/*
 * These two functions close the sound card handles
 */
  void
Close_Capture_Handle( void )
{
  if( capture_handle != NULL )
	snd_pcm_close( capture_handle );
  capture_handle = NULL;

  if( hw_params != NULL )
	snd_pcm_hw_params_free( hw_params );
  hw_params = NULL;
}

  void
Close_Mixer_Handle( void )
{
  if( mixer_handle != NULL )
	snd_mixer_close( mixer_handle );
  mixer_handle = NULL;
}

/*------------------------------------------------------------------------*/

/* Xrun_Recovery()
 *
 * Recover from underrun (broken pipe) and suspend
 */
  static gboolean
Xrun_Recovery( int error )
{
  if( error == -EPIPE )
  {
	error = snd_pcm_prepare( capture_handle );
	if( error < 0 )
	{
	  char mesg[MESG_SIZE];
	  snprintf( mesg, sizeof(mesg),
		  _("Cannot recover from underrun, prepare failed\n"\
			"Error: %s\n"), snd_strerror(error) );
	  Error_Dialog( mesg );
	  return( FALSE );
	}
  }
  else if( error == -ESTRPIPE )
  {
	while( (error = snd_pcm_resume(capture_handle)) == -EAGAIN )
	  sleep(1);
	if( error < 0 )
	{
	  error = snd_pcm_prepare( capture_handle );
	  if( error < 0 )
	  {
		char mesg[MESG_SIZE];
		snprintf( mesg, sizeof(mesg),
			_("Cannot recover from suspend, prepare failed\n"\
			"Error: %s\n"), snd_strerror(error) );
		Error_Dialog( mesg );
		return( FALSE );
	  }
	}
  }

  return( TRUE );
} /* Xrun_Recovery() */

/*------------------------------------------------------------------------*/

/*  Get_Signal_Sample()
 *
 *  Gets the next DSP sample of the signal input
 */

  gboolean
Signal_Sample( short *sample )
{
  snd_pcm_sframes_t error;

  static int
	dft_idx = 0, /* dft input buffer idx */
	cnt     = 0; /* Count of calls to this function */


  /* Refill dsp samples buffer when needed */
  if( samples_buffer.buffer_idx >= samples_buffer.buffer_size )
  {
	/* Start buffer index according to channel */
	samples_buffer.buffer_idx = rc_data.use_chnl;

	/* Read audio samples from DSP, abort on error */
	error = snd_pcm_readi(
		capture_handle, samples_buffer.buffer,	READI_FRAMES );
	if( error != READI_FRAMES )
	{
	  char mesg[MESG_SIZE];
	  snprintf( mesg, sizeof(mesg),
		  _("Read from audio interface failed\n"\
			"Error: %s\n"), snd_strerror((int)error) );
	  fprintf( stderr, "%s", mesg );

	  /* Try to recover if broken pipe or suspend */
	  if( !Xrun_Recovery((int)error) )
	  {
		Cleanup();
		return( FALSE );
	  }
	}
  } /* End of if( buffer_idx >= buffer_size ) */

  /* Get a sample from the buffer in short form */
  *sample = samples_buffer.buffer[samples_buffer.buffer_idx];

  /* Summate (decimate) samples for the DFT */
  dft_in_r[dft_idx] += *sample;

  /* Reset stride (decimation) counter */
  if( ++cnt >= rc_data.dft_stride )
  {
	cnt = 0;

	/* Normalize DFT input samples */
	dft_in_r[dft_idx] /= rc_data.dft_stride;
	dft_idx++;

	/* Display waterfall when input buffer full */
	if( dft_idx >= DFT_INPUT_SIZE )
	{
	  Display_Waterfall();
	  dft_idx = 0;
	}

	/* Clear for next summation */
	dft_in_r[dft_idx] = 0;
  }

  /* Increment according to mono/stereo mode */
  samples_buffer.buffer_idx += rc_data.num_chnl;

  return( TRUE );
} /* Signal_Sample() */

/*------------------------------------------------------------------------*/
